--
-- Guild Recruitment Helper
-- Help advertise recruiting information to trade and lfg channels
--
-- Author: janike
-- Version: r11
--
-- Thanks to
--	Isilorn for guild cooperation code and concept, and for frFR localization
--
--


-- debug mode variable
local debug = false


GuildRecr = LibStub("AceAddon-3.0"):NewAddon("GuildRecr", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceComm-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("GuildRecr", true)


--
-- Addon configuration
--

-- get option value
local function GetOption(info)
	return GuildRecr.db.global[info[#info]]
end

-- set option value
local function SetOption(info, value)
	GuildRecr.db.global[info[#info]] = value
end

-- safe cut utf8 string
local function utf8cut(value, len)
	if type(value) == "string" and #value > 0 and len >= 1 then
		if #value > len then
			if value:byte(len) >= 192 then -- UTF-8 multibyte first byte
				value = value:sub(1, len - 1)
			elseif value:byte(len - 1) >= 224 then -- UTF-8 triplebyte first byte
				value = value:sub(1, len - 2)
			else
				value = value:sub(1, len)
			end
		end
	end
	
	return value
end

-- update message
local function SetMessage(info, value)
	if not value then
		self.db.global.message = ""
		return
	end
	
	-- split message
	local msg = { strsplit("\n", strtrim(value) .. "\n") }
	
	-- only 2 lines allowed of 255 bytes each
	local l1 = utf8cut(strtrim(msg[1]), 255)
	local l2 = utf8cut(strtrim(msg[2]), 255)
	
	GuildRecr.db.global[info[#info]] = strjoin("\n", l1, l2)
end

-- update announce timer
local function UpdateTimer(info, value)
	-- update timer config
	GuildRecr.db.global[info[#info]] = value

	-- reschedule timer
	GuildRecr:SetTimer()
end


-- declare defaults to be used in the DB
local defaults = {
	realm = {
		enabled = false,
	},
	global = {
		timer = 5,
		min_lines = 100,
		announce_trade = false,
		announce_lfg = true,
		message = "",
		set_dnd = false,
		dnd_message = "",
		guild_coop = true,
		test_mode = false,
	}
}

-- declare config interface
local options = {
    name = "Guild Recruitment",
    handler = GuildRecr,
    type = "group",
	childGroups = "tab",
    args = {
		general_tab = {
			name = "General",
			type = "group",
			order = 10,
			args = {
				enabled = {
					type = "toggle",
					order = 11,
					name = L["Enable Addon"],
					desc = L["Enable or disable addon functionality."],
					width = "full",
					get =	function ()
								return GuildRecr.db.realm.enabled
							end,
					set =	function (info, value)
								GuildRecr.db.realm.enabled = value
								GuildRecr:SetTimer()
							end,
				},
				description = {
					type = "description",
					order = 12,
					fontSize = "medium",
					name = L["addon_description"],
				},
			},
		},
		message_tab = {
			name = "Message",
			type = "group",
			order = 20,
			args = {
				message = {
					type = "input",
					order = 21,
					name = L["Message to announce, 2 lines max"],
					desc = L["Type the message to announce, every line will be announced. Message should not be longer than 2 lines."],
					multiline = 7,
					width = "full",
					get = GetOption,
					set = SetMessage,
				},
				timer = {
					type = "range",
					order = 22,
					name = L["Announce interval, minutes"],
					desc = L["How often announce your message to channel."],
					min = 3,
					max = 30,
					step = 1,
					get = GetOption,
					set = UpdateTimer,
				},
				min_lines = {
					type = "range",
					order = 23,
					name = L["SPAM protection, lines"],
					desc = L["This add-on will try to prevent spamming to idle channels. If there is no activity in a channel, then it waits a little longer. Specify how many lines should pass before add-on is allowed to announce message."],
					min = 50,
					max = 150,
					step = 10,
					get = GetOption,
					set = SetOption,
				},
				announce_trade = {
					type = "toggle",
					order = 24,
					name = L["Announce to Trade"],
					desc = L["Announce your message to city trade channel."],
					get = GetOption,
					set = SetOption,
				},
				announce_lfg = {
					type = "toggle",
					order = 25,
					name = L["Announce to LFG"],
					desc = L["Announce your message to LFG channel."],
					get = GetOption,
					set = SetOption,
				},
				guild_coop = {
					type = "toggle",
					order = 26,
					name = L["Guild cooperation mode"],
					desc = L["Doesn't allow multiple announces from your guild members at the same time."],
					width = "full",
					get = GetOption,
					set = SetOption,
				},
				test_mode = {
					type = "toggle",
					order = 27,
					name = L["Test mode"],
					desc = L["Doesn't send anything to channels, simulating only."],
					width = "full",
					get = GetOption,
					set = SetOption,
				},
			},
		},
		status_tab = {
			name = "Status",
			type = "group",
			order = 50,
			args = {
				set_dnd = {
					type = "toggle",
					order = 51,
					name = L["Set DND"],
					desc = L["Always set your DND status while announcing to channels"],
					width = "full",
					get = GetOption,
					set = 	function (info, value)
								GuildRecr.db.global.set_dnd = value
								if not value then GuildRecr:SetDND(true) end -- clear dnd
							end,
				},
				dnd_msg = {
					type = "input",
					order = 52,
					name = L["Set DND message"],
					desc = L["Type DND message there."],
					width = "full",
					get = GetOption,
					set = 	function (info, value)
								GuildRecr.db.global.dnd_msg = strtrim(value)
								GuildRecr:SetDND(true) -- clear dnd
							end,
				},
			},
		},
    },
}


--
-- Addon initialization
--

-- Code that you want to run when the addon is first loaded goes here.
function GuildRecr:OnInitialize()
    -- initialize saved variables
	self.db = LibStub("AceDB-3.0"):New("GuildRecrDB", defaults, true)
	
	-- setup variables
	self:InitVars()
	
	-- initialize configuration options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("GuildRecr", options)
	self.configFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("GuildRecr", "Guild Recruitment");
	
	-- create LibDataBroker
	self.ldb = LibStub("LibDataBroker-1.1"):NewDataObject("Guild Recruitment", {
		type = "data source",
		text = "",
		label = "",
		icon = "Interface\\Icons\\INV_Shirt_GuildTabard_01",
		OnClick =	function (frame, button)
						if button == "LeftButton" then
							-- update config
							GuildRecr.db.realm.enabled = not GuildRecr.db.realm.enabled
							LibStub("AceConfigRegistry-3.0"):NotifyChange("GuildRecr")

							-- reschedule timer
							GuildRecr:SetTimer()
						else
							-- RightButton: options menu
							InterfaceOptionsFrame_OpenToCategory(self.configFrame)
						end
					end,
		OnTooltipShow =	function(tooltip)
							tooltip:AddLine("Guild Recruitment")
							tooltip:AddLine(" ")
							tooltip:AddLine(L["Left-click to toggle ON/OFF"], 0, 1, 0)
							tooltip:AddLine(L["Right-click to open the configuration"], 0, 1, 0)
						end,
		})
		
		
end

-- Called when the addon is enabled
function GuildRecr:OnEnable()
	-- register events
	self:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE", "EventChannelNotice")
	self:RegisterEvent("CHAT_MSG_CHANNEL", "EventChannelMsg")
	
	-- register comm events
	self:RegisterComm("GuildRecrAnnounce", "CommAnnounce")
	
	-- register console commands
	self:RegisterChatCommand("guildrecr", "ConsoleCommand")
	self:RegisterChatCommand("grec", "ConsoleCommand")
	self:RegisterChatCommand("gr", "ConsoleCommand")
	
	-- schedule timer
	self:SetTimer()
end

-- Called when the addon is disabled
function GuildRecr:OnDisable()
	-- unregister events
	self:UnregisterAllEvents()
	
	-- unregister comm events
	self:UnregisterAllComm()
	
	-- unregister console commands
	self:UnregisterChatCommand("guildrecr")
	self:UnregisterChatCommand("grec")
	self:UnregisterChatCommand("gr")
	
	-- cancel timers
	self:CancelAllTimers()
end

-- Initalize local variables
function GuildRecr:InitVars()
	-- trade and lfg channels id
	self.trade_id = nil
	self.lfg_id = nil
	
	-- channel msg counters
	self.trade_cnt = 0
	self.lfg_cnt = 0
	
	-- skip counter
	self.skip_cnt = 0
	
	-- guild coop mode, last advertised player name
	self.guild_coop_by = nil
	
	-- timer handle
	self.timer = nil
end

-- Setup announce timer
function GuildRecr:SetTimer()
	-- set ldb text
	self.ldb.text = L["Off"]
	self.ldb.label = self.ldb.text
	
	-- cancel running timer if any
	if self.timer then
		self:CancelTimer(self.timer, false)
		self.timer = nil		
	end
	
	-- schedule new timer
	if self.db.realm.enabled then
		self.timer = self:ScheduleRepeatingTimer("TimerAnnounce", self.db.global.timer * 60)
		
		-- update ldb text
		self.ldb.text = format(L["On (%d min)"], self.db.global.timer)
		self.ldb.label = self.ldb.text
	end
end

-- set DND status if requested
function GuildRecr:SetDND(clear)
	-- clear DND status
	if clear and UnitIsDND("player") then
		SendChatMessage("", "DND")
		return
	end
	
	-- set DND if required
	if clear or not self.db.global.set_dnd or not self.db.global.dnd_msg then
		return
	end
	
	if not UnitIsDND("player") then
		SendChatMessage(self.db.global.dnd_msg, "DND")
	end
end


--
-- Event Handlers
--

-- Fired when you enter or leave a chat channel
function GuildRecr:EventChannelNotice(event, arg1, _, _, _, _, _, arg7, arg8)
	-- arg7 channel type (2 - Trade, 26 - LFG)
	-- arg8 channel number
	
	-- Trade channel
	if arg7 and arg7 == 2 then
		if arg1 == "YOU_JOINED" then
			self.trade_id = arg8
		else
			self.trade_id = nil
		end
	end

	if arg7 and arg7 == 26 then
		self.lfg_id = nil

		if arg1 == "YOU_JOINED" then
			self.lfg_id = arg8
		else
			self.lfg_id = nil
		end
	end
end

-- Fired when the client receives a channel message.
function GuildRecr:EventChannelMsg(event, _, _, _, _, _, _, arg7, arg8)
	-- fix reload: rejoin channels if message received on that channel
	if arg7 and arg7 == 2 then
		self.trade_id = arg8
	end

	if arg7 and arg7 == 26 then
		self.lfg_id = arg8
	end

	-- count trade messages
	if self.trade_id and arg8 == self.trade_id then
		self.trade_cnt = self.trade_cnt + 1
	end

	-- count lfg messages
	if self.lfg_id and arg8 == self.lfg_id then
		self.lfg_cnt = self.lfg_cnt + 1
	end
end

-- Announce timer handler
function GuildRecr:TimerAnnounce()
	-- check channel availability and possibility to announce
	local can_trade	= self.trade_id and self.db.global.announce_trade and (self.trade_cnt >= self.db.global.min_lines or self.skip_cnt >= 2)
	local can_lfg	= self.lfg_id and self.db.global.announce_lfg and (self.lfg_cnt >= self.db.global.min_lines or self.skip_cnt >= 2)
	
	-- guild coop check
	if self.guild_coop_by then
		self:Printf(L["Skipping. Message already advertised by %s"], self.guild_coop_by)
		self.guild_coop_by = nil
		self.trade_cnt = 0
		self.lfg_cnt = 0
		self.skip_cnt = 0
		return
	end

	-- determine most active channel if both available
	if can_trade and can_lfg then
		if self.trade_cnt > self.lfg_cnt then
			can_lfg = false
		else
			can_trade = false
		end
	end
	
	-- announce to trade channel
	if can_trade then
		self:Printf(L["Announcing to Trade channel. Trade: %d LFG: %d"], self.trade_cnt, self.lfg_cnt)
		self.trade_cnt = 0
		self:Announce(self.trade_id)
		return
	end
	
	-- announce to lfg channel
	if can_lfg then
		self:Printf(L["Announcing to LFG channel. Trade: %d LFG: %d"], self.trade_cnt, self.lfg_cnt)
		self.lfg_cnt = 0
		self:Announce(self.lfg_id)
		return
	end
	
	-- requirements not meet, print stats
	if self.trade_id or self.lfg_id then
		self:Printf(L["Skipping. Trade: %d LFG: %d"], self.trade_cnt, self.lfg_cnt)
		self.skip_cnt = self.skip_cnt + 1
	end
end

-- send message to specified channel
function GuildRecr:Announce(channel)
	if not channel or not self.db.global.message or not strtrim(self.db.global.message) then
		return
	end
	
	-- reset skip counter
	self.skip_cnt = 0
	
	-- split message and announce
	local msg = { strsplit("\n", strtrim(self.db.global.message)) }
	
	if not #msg[1] then
		return
	end
	
	-- output 2 lines max
	for i = 1, 2 do
		if msg[i] and strtrim(msg[i]) ~= "" then
			local line = strtrim(msg[i])
			
			-- send message
			if #line then
				if self.db.global.test_mode then
					self:Print(L["Test mode"] .. ": " .. line)
				else
					SendChatMessage(line, "CHANNEL", nil, channel)
				end
			end
		end
	end

	-- announce guild event if needed
	if self.db.global.guild_coop and IsInGuild() then
		self:SendCommMessage("GuildRecrAnnounce", UnitName("player"), "GUILD")
	end
	
	-- set DND if needed
	self:SetDND()
end

-- Fired when another player on your guild announce message to channels
function GuildRecr:CommAnnounce(prefix, message, distribution, sender)
	-- should we care?
	if self.db.global.guild_coop and sender and sender ~= UnitName("player") then
		self.guild_coop_by = sender
	end
end


--
-- Console commands
--

-- Process the slash command ("input" contains whatever follows the slash command)
function GuildRecr:ConsoleCommand(input)
	-- show configuration window if no params given
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.configFrame)
	end
	
	if input == "debug" then
		self:Print("Debug mode enabled")
		debug = true
	end

	if input == "nodebug" then
		self:Print("Debug mode disabled")
		debug = false
	end
end